#!-PERL-
# mif2fm.pl
#
# THIS FILE IS COPYRIGHT 1994 BY TELENOR RESEARCH, NORWAY
#
#                    ALL RIGHTS RESERVED.
#
# For rights granted to the public see the COPYRIGHT file
#
# Programming : Jon Stephenson von Tetzchner

# This script is part of the fm2html package.

$\ = "\n";		# set output record separator
$, = " ";		# set output field separator
$* = 0;			# set single line pattern matching

select( STDERR );	# all error messages go to stderr
$| = 1;			# make stderr unbuffered

$FALSE = 0;
$TRUE  = 1;

$FM_MAGIC	= "<MakerFile";
$FM_BOOK_MAGIC	= "<Bookfile";
$MIF_MAGIC	= "<MIFFile";
$MIF_BOOK_MAGIC	= "<Book ";

$TMPNAME	= "mif2fm.tmp";


sub escape	# take string, insert backslash before special characters
{
  local( $_ ) = @_;
  s|([\.\+\?\*\'\"\(\)])|\\$1|g;
  $_;
}

sub dirname	# take pathname, return leading directories
{
  local( $_ ) = @_;
  s|^(.+)/.*$|$1| || s|^/[^/]*$|/| || s|^[^/]*$|.|;
  $_;
}

sub basename	# take pathname, strip leading directories and optional suffix
{
  local( $_, $suffix ) = @_;
  m|/|o && s|^.*/([^/]*)$|$1|;
  $suffix = &escape( $suffix );
  s|(.*)$suffix|$1|;
  $_;
}


local( $prog ) = &basename( $0 );
local( $usage ) =  "Usage: $prog [-q] [-r] filename";

local( $infile, $indir, $cwd, $tmpfile, $script, $outfile, $fmbatch, 
       $version, $execpath, $mif, $doc, @inputs, @extras );

local( $quiet ) = $FALSE;
local( $remove ) = $FALSE;
local( $redirect ) = "1>&2";
local( $exitval ) = 0;

while ( $ARGV[0] =~ m|^-| )
{
  $_ = shift;
  last if m|^--|;
  
  if ( m|^-q$| )
  {
    $quiet = $TRUE; next;
  }
  elsif ( m|^-r$| )
  {
    $remove = $TRUE; next;
  }

  print "$prog: ERROR: Invalid flag: \"$_\"";
  print "$usage";
  exit( 1 );
}

if ( $#ARGV != 0 )
{
  print "";
  print "$usage";
  print "";
  print "\tThe parameter specifies the name of a FrameMaker MIF file";
  print "\tto translate to a binary FrameMaker document file.  If a";
  print "\tFrameMaker book file is specified then all files in the";
  print "\tbook will be translated, as will the book itself.";
  print "";
  print "\tThe name of each output file is formed by appending";
  print "\t\".doc\" to the name of the input file.  If the name of";
  print "\tan input file ends in a \".mif\" extension, that extension";
  print "\twill be removed prior to \".doc\" being appended.";
  print "";
  print "\tIf a binary document file exists already then it will be";
  print "\trenamed to have a \".orig\" extension.  If a \".orig\" file";
  print "\texists already then the extension \".bak\" will be used instead.";
  print "";
  print "\tSpecifying \"-r\" causes the input MIF file(s) to be removed";
  print "\tonce they have been converted.  As well, any \".orig\"/\".bak\"";
  print "\tfile(s) will be deleted.";
  print "";
  print "\tSpecifying \"-q\" causes the program to operate in \"quiet\"";
  print "\tmode, in which messages are printed only if errors occur.";
  print "";
  exit( 1 );
}

$infile = "$ARGV[0]";

if ( ! -e "$infile" )
  { die "$prog: ERROR: Input file \"$infile\" does not exist... exiting"; }
if ( ! open( INFILE, "<$infile" ) )
  { die "$prog: ERROR: Could not open input file \"$infile\"... exiting"; }

$idline = <INFILE>;

if ( $idline =~ m|^$FM_MAGIC|i || $idline =~ m|^$FM_BOOK_MAGIC|i )
  { die "$prog: ERROR: File \"$infile\" is already in FrameMaker document format... exiting"; }
if ( $idline !~ m|^$MIF_MAGIC|i && $idline !~ m|^$MIF_BOOK_MAGIC|i )
  { die "$prog: ERROR: File \"$infile\" is not a MIF file... exiting"; }

# djf: Docs and xrefs may have relative pathnames so it is necessary to chdir.
# Unfortunately, however, FrameMaker sometimes has problems figuring out
# what the current directory is and defaults to the user's home directory,
# in which case it won't find the files anyway.
$indir = &dirname( "$infile" );
chdir( "$indir" ) ||
  die "mif2mif: ERROR: Can't change directory to \"$indir\"... exiting";
chop( $cwd = `pwd` );
$infile = &basename( "$infile" );
$infile = "$cwd/$infile";

if ( $quiet == $TRUE )
{
  local( $tmpdir ) = $ENV{'TMPDIR'};

  if ( ! $tmpdir ) { $tmpdir = "-TMPDIR-"; }
  if ( ! $tmpdir ) { $tmpdir = "$cwd"; }
  $tmpdir =~ s|/$||;
  $tmpfile = "$tmpdir/$TMPNAME.$$";
  $redirect = "1>$tmpfile 2>&1";
}

$idline =~ m|^\S+\s+(.)|;
$version = $1;

if ( ! "-FMBATCH4-" )
{
  $fmbatch = "-FMBATCH3-";

  if ( $version > 3 )
    { $quiet == $FALSE && print "$prog: WARNING: FrameMaker batch mode program \"$fmbatch\" will convert version $version MIF files to version 3 FrameMaker documents"; }
}
elsif ( ! "-FMBATCH3-" )
{
  $fmbatch = "-FMBATCH4-";

  if ( $version < 4 )
    { $quiet == $FALSE && print "$prog: WARNING: FrameMaker batch mode program \"$fmbatch\" will convert version $version MIF files to version 4 FrameMaker documents"; }
}
elsif ( $version < 4 )
{
  $fmbatch = "-FMBATCH3-";
}
else
{
  $fmbatch = "-FMBATCH4-";
}

$execpath = "$fmbatch";
$execpath =~ s|^(\S+)\s*.*|$1|;

if ( ! -x "$execpath" )
  { die "$prog: ERROR: FrameMaker batch mode program \"$execpath\" is not executable... exiting"; }

if ( $idline =~ m|^$MIF_BOOK_MAGIC|i )
{
  local( @mifs, $oldfile );

  $quiet == $FALSE && print "\n$prog: Converting book file \"$infile\"...";
  
  while ( <INFILE> )
  {
    if ( m|<FileName | )
    {
      local( $docpath, $mifpath, $suffix );

      if ( m|<FileName \s*`<c\\>[^<]*<U\\>([^']*)'>| )
      {
        $docpath = "$1";	# djf: FrameMaker 3.1
      }
      elsif ( m|<FileName \s*`<c\\>([^<]*)'>| )
      {
        $docpath = "$1";	# djf: FrameMaker 4.0
      }

      $mifpath = "$docpath.mif";

      if ( ! -e "$mifpath" )
      {
	local( $unstripped );
	$unstripped = "$mifpath";
	$mifpath =~ s|\.doc||;

	if ( "$mifpath" eq "$unstripped" )
	{
	  print "$prog: ERROR: Input file \"$unstripped\" does not exist";
	  $exitval = 1;
	  next;
	}
	elsif ( ! -e $mifpath )
	{
	  print "$prog: ERROR: Input file \"$unstripped\" does not exist, neither does \"$mifpath\"";
	  $exitval = 1;
	  next;
	}
      }

      $mifs[++$#mifs] = "$mifpath";
    }
  }

  @ARGV = @mifs;

  $mif = "$infile";
  $extras[++$#extras] = "$mif";
  $doc = "$mif";
  $doc =~ s|\.mif$||;
  "$doc" eq "$mif" && ($doc = "$mif.doc");

  if ( -e $doc )
  {
    local( $orig );

    $orig = ( -e "$doc.orig" ) ? "$doc.bak" : "$doc.orig"; 
    rename( "$doc", "$orig" ) && ($quiet == $FALSE) && 
      (print "$prog: Existing file \"$doc\" renamed as \"$orig\"...");
    $extras[++$#extras] = "$orig";
  }

  $script .= <<END;
Open $mif
SaveAs d $mif $doc
END
}
else
{
  $ARGV[0] = "$infile";
}

close( INFILE );

while ( $mif = shift )
{
  $doc = "$mif";
  $doc =~ s|\.mif$||;
  "$doc" eq "$mif" && ($doc = "$mif.doc");

  if ( ! open( MIF, "<$mif" ) )
  {
    print "$prog: ERROR: Can't open file \"$mif\"";
    $exitval = 1;
    next;
  }

  $idline = <MIF>;
  close( MIF );

  if ( $idline =~ m|^$FM_MAGIC|i || $idline =~ m|^$FM_BOOK_MAGIC|i )
  {
    $quiet == $FALSE && print
      "$prog: ATTENTION: File \"$mif\" is already in FrameMaker document f
ormat";
    next;
  }

  if ( $idline !~ m|^$MIF_MAGIC|i && $idline !~ m|^$MIF_BOOK_MAGIC|i )
  {
    print "$prog: ERROR: File \"$mif\" is not a FrameMaker file";
    $exitval = 1;
    next;
  }

  if ( -e $doc )
  {
    local( $orig );

    $orig = ( -e "$doc.orig" ) ? "$doc.bak" : "$doc.orig"; 
    rename( "$doc", "$orig" ) && ($quiet == $FALSE) &&
      (print "$prog: Existing file \"$doc\" renamed as \"$orig\"...");
    $extras[++$#extras] = "$orig";
  }

  $script .= <<END;
Open $mif
SaveAs d $mif $doc
END

  $inputs[++$#inputs] = "$mif";
  $extras[++$#extras] = "$mif";
}

$script .= <<END;
Quit
END

if ( $quiet == $FALSE )
{
  local( $name );
  print "\n$prog: Converting the following MIF files to FrameMaker documents:";
  while ( $name = shift( @inputs ) )
    { print "  $name"; }
}

$quiet == $FALSE && print "";

eval open( FMBATCH, "| $fmbatch $redirect" );
print FMBATCH "$script";
close( FMBATCH );

if ( $? != 0 )
{
  if ( $quiet == $TRUE )
  {
    # djf: It would be nice to eliminate the need for $tmpfile by piping
    # fmbatch output directly to grep.  However, if you do that then the
    # status value returned by "close( FMBATCH )" is for the grep, not the
    # fmbatch job.
    eval system( "grep -i error $tmpfile 1>&2" );
    eval system( "rm -f $tmpfile 1>&2" );
  }
  die "$prog: ERROR: FrameMaker batch program \"$fmbatch\" failed... exiting";
}

eval system( "rm -f $tmpfile 1>&2" );

if ( $remove == $TRUE )
{
  local( $extras ) = join( ' ', @extras );
  eval system( "rm -f $extras 1>&2" );
}

$quiet == $FALSE && print "\n$prog: Done\n";

exit( $exitval );
